' History Dump Script (iehistory2.vbs)
' By: Ed Levin
' 8 / 1 /06
'
' DESCRIPTION: This script dumps a list of unique addresses from history folders of all users having profiles on the computer into a text file,
' listing one address per line. It utilizes CLI functionality of NirSoft's IE History View v1.33 to retrieve history from user profiles.
'
' ARGUMENTS: The script accepts 1 argument which is the path for the output text file and the IEHV executable.
'
' REQUIREMENTS: The script has to be executed locally and iehv.exe needs to be present in the specified path.
'
' PURPOSE: The script is meant to be executed from a Kaseya script, "Retrieve IE History Addresses v2", to return browsed address history list from a remote
' machine to the server for further processing (Emailing the list or having it consumed by another program or script).

Const ForReading = 1
Const Overwrite = TRUE
Const WaitOnReturn = TRUE
Const INPUT_FILE = "rawhistory.txt"
Const OUTPUT_FILE = "history.txt"
Const IEHV_EXE = "iehv.exe"
Const GENERAL_FAILURE = 2
Const HKLM = &H80000002
Const sProfileRegBase = "SOFTWARE\Microsoft\Windows NT\CurrentVersion\ProfileList"

Set ArgObj = WScript.Arguments

If ArgObj.Count <> 1 Then
    Wscript.Echo "Incorrect number of arguments provided."
	Wscript.Quit(GENERAL_FAILURE)
End If

Dim arrUniqueURL(), arrUniqueProfilePath()

' Set filesystem variables
localPath = argObj(0)
outputFile = localPath & "\" & OUTPUT_FILE
inputFile = localPath & "\" & INPUT_FILE
iehvProgram = localPath & "\" & IEHV_EXE

' Initialize objects and open output file for writing
Set objFSO = CreateObject("Scripting.FileSystemObject")
Set objURLHistoryFile = objFSO.CreateTextFile(outputFile,Overwrite)
Set oWshNet = CreateObject("Wscript.Network") 
sComputer = oWshNet.ComputerName
On Error Resume Next
Set objReg = GetObject("WinMgmts:{impersonationLevel=impersonate}!//" _ 
                & sComputer & "/root/default:StdRegProv")
If Err.Number <> 0 Then 
    objURLHistoryFile.WriteLine VbCrLf & "Error accessing WMI registry provider."
    objURLHistoryFile.WriteLine "Error: " & err.Description
    objURLHistoryFile.WriteLine "Code:  " & err.Number & VbCrLf
End If 
On Error GoTo 0
                
Set Command = WScript.CreateObject("WScript.Shell")

' Output file header
objURLHistoryFile.WriteLine "USER URL ADDRESS HISTORY FOR COMPUTER: " & sComputer

' Enumerate ProfileList registry subkeys
On Error Resume Next
objReg.EnumKey HKLM, sProfileRegBase, arrSubKeys
If Err.Number <> 0 Then 
    objURLHistoryFile.WriteLine VbCrLf & "Error enumerating profile registry keys under: HKLM\" & sProfileRegBase
    objURLHistoryFile.WriteLine "Error: " & err.Description
    objURLHistoryFile.WriteLine "Code:  " & err.Number & VbCrLf
End If 
On Error GoTo 0

' Reset profile path duplicates array
ReDim arrUniqueProfilePath(0)

' Process profile paths
For Each objItem In arrSubKeys

	' Exclude certain well-known yet irrelevant profiles (Local System, Local Service, Network Service)
	If (StrComp(objItem,"S-1-5-18") <> 0) AND (StrComp(objItem,"S-1-5-19") <> 0) AND (StrComp(objItem,"S-1-5-20") <> 0) Then

		' Get account profile path from the registry
		On Error Resume Next
		objReg.GetExpandedStringValue HKLM, sProfileRegBase & "\" & objItem, "ProfileImagePath", sProfilePath
		If Err.Number <> 0 Then 
    		objURLHistoryFile.WriteLine VbCrLf & "Error retrieving profile path from registry for: " & objItem
    		objURLHistoryFile.WriteLine "Error: " & err.Description
    		objURLHistoryFile.WriteLine "Code:  " & err.Number & VbCrLf
		End If 
		On Error GoTo 0

		' Check if the path exists for the current user instance
		If IsNull(sProfilePath) Then
		Else
			
			Match = False
			
			' Test for a duplicate paths
       		For i = 0 to UBound(arrUniqueProfilePath)
        		If strComp(sProfilePath, arrUniqueProfilePath(i)) <> 0 Then
         		Else
     	 			Match = True
      	 			Exit For
        		End If  	
       		Next
				
			' If no duplicate found, add to duplicates array
       		If Match = False Then
      			arrUniqueProfilePath(Ubound(arrUniqueProfilePath)) = sProfilePath
      			ReDim Preserve arrUniqueProfilePath(UBound(arrUniqueProfilePath) + 1)
       		End If
       	End If
  	End If
Next

' Trim a little off the top
ReDim Preserve arrUniqueProfilePath(UBound(arrUniqueProfilePath) - 1)

For Each objItem In arrUniqueProfilePath
			
	' Run IEHistoryView on the current user
	iehvCommand = iehvProgram & " /stabular """ & inputFile & """ -folder " & """" & objItem & "\Local Settings\History" & """"
	ReturnCode = Command.Run(iehvCommand,1,WaitOnReturn)

	' Open IEHV result file for parsing
	Set objSourceFile = objFSO.OpenTextFile(inputFile, ForReading)

	' Output current user instance header
	objURLHistoryFile.WriteLine VbCrLf & "PROFILE PATH: " & objItem & VbCrLf

    ' Reset URL duplicates array
	ReDim arrUniqueURL(0)
    			
    ' Parse file one line at a time
	Do Until objSourceFile.AtEndOfStream
    	urlItem = objSourceFile.Readline

       	' Make sure the entry is an HTTP address
       	If StrComp(Left(Ucase(urlItem),4),"HTTP") = 0 Then
       				
       		' Extract the address part from the URL
       		arrSplitSlash = Split(urlItem , "/",4)
       		arrSplitColon = Split(arrSplitSlash(2),":",2)
       		arrSplitSpace = Split(arrSplitColon(0), " ",2)
       		URL = arrSplitSpace(0)
       					
       		Match = False
      
	   		' Test for a duplicate address
       		For i = 0 to UBound(arrUniqueURL)
        		If strComp(URL, arrUniqueURL(i)) <> 0 Then
         		Else
     	 			Match = True
      	 			Exit For
        		End If  	
       		Next
       
	   		' If no duplicate found, write to file, add to duplicates array
       		If Match = False Then
        		objURLHistoryFile.WriteLine URL
      			ReDim Preserve arrUniqueURL(UBound(arrUniqueURL) + 1)
      			arrUniqueURL(Ubound(arrUniqueURL)) = URL
       		End If
       	End If	
     Loop
			
     'Delete IEHV results file
     objSourceFile.Close()
     objFSO.DeleteFile inputFile
     
Next

objURLHistoryFile.Close()